Attribute VB_Name = "modRotateImage"
'********************************************
'* Module to print rotated image on screen: *
'********************************************

Option Explicit

Private Type POINTAPI

  X As Long
  Y As Long
  
End Type

'Declares used by DrawStdPictureRot:
Private Declare Function PlgBlt Lib "gdi32.dll" (ByVal hdcDest As Long, ByRef lpPoint As POINTAPI, ByVal hdcSrc As Long, ByVal nXSrc As Long, ByVal nYSrc As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hbmMask As Long, ByVal xMask As Long, ByVal yMask As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32.dll" (ByVal hdc As Long) As Long
Private Declare Function SelectObject Lib "gdi32.dll" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteDC Lib "gdi32.dll" (ByVal hdc As Long) As Long

Public Sub RotateImage(ByRef frm As Form, ByRef img As Image, ByRef ori As Image, ByRef pic1 As PictureBox, ByRef pic2 As PictureBox, ByVal iRotation As Integer)

  On Error Resume Next
  img.Visible = False
  pic1.Picture = ori.Picture
  pic2.Width = pic1.Width
  pic2.Height = pic1.Height
  pic2.AutoRedraw = True
  pic2.Cls
  DrawStdPictureRot pic2.hdc, 0, 0, -iRotation, pic1.Picture, frm
  pic2.Refresh
  img.Picture = pic2.Image
  img.Visible = True
  
End Sub

Private Function DrawStdPictureRot(ByVal inDC As Long, ByVal inX As Long, ByVal inY As Long, ByVal inAngle As Single, ByRef inPicture As StdPicture, ByRef inForm As Form) As Long

  On Error Resume Next
  
  Dim hdc As Long
  Dim hOldBMP As Long
  Dim PlgPts(0 To 4) As POINTAPI
  Dim PicWidth As Long, PicHeight As Long
  Dim HalfWidth As Single, HalfHeight As Single
  Dim AngleRad As Single
  
  Const PI As Single = 3.14159
  Const HalfPi As Single = PI * 0.5
  
  ' Validate input picture
  If (inPicture Is Nothing) Then Exit Function
  If (inPicture.Type <> vbPicTypeBitmap) Then Exit Function
  
  ' Get picture size
  PicWidth = inForm.ScaleX(inPicture.Width, vbHimetric, vbPixels)
  PicHeight = inForm.ScaleY(inPicture.Height, vbHimetric, vbPixels)
  
  ' Get half picture size and angle in radians
  HalfWidth = PicWidth / 2
  HalfHeight = PicHeight / 2
  AngleRad = (inAngle / 180) * PI
  
  ' Create temporary DC and select input picture into it
  hdc = CreateCompatibleDC(0&)
  hOldBMP = SelectObject(hdc, inPicture.handle)
  
  If (hOldBMP) Then ' Get angle vectors for width and height
  
    PlgPts(0).X = Cos(AngleRad) * HalfWidth
    PlgPts(0).Y = Sin(AngleRad) * HalfWidth
    PlgPts(1).X = Cos(AngleRad + HalfPi) * HalfHeight
    PlgPts(1).Y = Sin(AngleRad + HalfPi) * HalfHeight
    
    ' Project parallelogram points for rotated area
    PlgPts(2).X = HalfWidth + inX - PlgPts(0).X - PlgPts(1).X
    PlgPts(2).Y = HalfHeight + inY - PlgPts(0).Y - PlgPts(1).Y
    PlgPts(3).X = HalfWidth + inX - PlgPts(1).X + PlgPts(0).X
    PlgPts(3).Y = HalfHeight + inY - PlgPts(1).Y + PlgPts(0).Y
    PlgPts(4).X = HalfWidth + inX - PlgPts(0).X + PlgPts(1).X
    PlgPts(4).Y = HalfHeight + inY - PlgPts(0).Y + PlgPts(1).Y
    
    ' Draw rotated image
    DrawStdPictureRot = PlgBlt(inDC, PlgPts(2), hdc, 0, 0, PicWidth, PicHeight, 0&, 0, 0)
    
    ' De-select Bitmap from DC
    SelectObject hdc, hOldBMP
    
  End If
  
  ' Destroy temporary DC
  DeleteDC hdc
  
End Function
